@@ -9,6 +9,7 @@ from GameSentenceMiner.util import ffmpeg, notification
 from GameSentenceMiner.util.configuration import gsm_state, logger, get_config, get_temporary_directory
 from GameSentenceMiner.util.ffmpeg import get_video_timings
 from GameSentenceMiner.util.text_log import GameLine
+from GameSentenceMiner.util.audio_player import AudioPlayer
 
 
 def set_get_audio_from_video_callback(func):
@@ -16,41 +17,91 @@ def set_get_audio_from_video_callback(func):
     get_audio_from_video = func
 
 
+# Global audio player instance
+_audio_player = None
+
+
+def get_audio_player():
+    """Get or create the global audio player instance."""
+    global _audio_player
+    if _audio_player is None:
+        _audio_player = AudioPlayer(finished_callback=_on_audio_finished)
+    return _audio_player
+
+
+def _on_audio_finished():
+    """Callback when audio playback finishes."""
+    # Clear the current audio stream reference from gsm_state
+    gsm_state.current_audio_stream = None
+
+
+def stop_current_audio():
+    """Stop the currently playing audio."""
+    player = get_audio_player()
+    player.stop_audio()
+    gsm_state.current_audio_stream = None
+
+
+def play_audio_data_safe(data, samplerate):
+    """
+    Play audio data using the safe audio player.
+    
+    Args:
+        data: Audio data as numpy array
+        samplerate: Sample rate of the audio
+        
+    Returns:
+        True if playback started successfully, False otherwise
+    """
+    player = get_audio_player()
+    success = player.play_audio_data(data, samplerate)
+    if success:
+        # Store reference in gsm_state for compatibility
+        gsm_state.current_audio_stream = player.current_audio_stream
+    return success
+
+
 def handle_texthooker_button(video_path=''):
     try:
         if gsm_state.line_for_audio:
+            # Check if audio is currently playing and stop it
+            if gsm_state.current_audio_stream:
+                stop_current_audio()
+                gsm_state.line_for_audio = None
+                return
+                
             line: GameLine = gsm_state.line_for_audio
             gsm_state.line_for_audio = None
+            
             if line == gsm_state.previous_line_for_audio:
                 logger.info("Line is the same as the last one, skipping processing.")
-                if get_config().advanced.audio_player_path:
-                    play_audio_in_external(gsm_state.previous_audio)
-                elif get_config().advanced.video_player_path:
+                if get_config().advanced.video_player_path:
                     play_video_in_external(line, video_path)
                 else:
-                    import sounddevice as sd
-                    data, samplerate = gsm_state.previous_audio
-                    sd.play(data, samplerate)
-                    sd.wait()
+                    # Use cached audio data with safe playback
+                    if gsm_state.previous_audio:
+                        data, samplerate = gsm_state.previous_audio
+                        play_audio_data_safe(data, samplerate)
                 return
+                
             gsm_state.previous_line_for_audio = line
-            if get_config().advanced.audio_player_path:
-                audio = get_audio_from_video(line, line.next.time if line.next else None, video_path,
-                                             temporary=True)
-                play_audio_in_external(audio)
-                gsm_state.previous_audio = audio
-            elif get_config().advanced.video_player_path:
+            
+            if get_config().advanced.video_player_path:
                 play_video_in_external(line, video_path)
             else:
-                import sounddevice as sd
+                # Extract audio and play with safe method
                 import soundfile as sf
                 audio = get_audio_from_video(line, line.next.time if line.next else None, video_path,
                                              temporary=True)
                 data, samplerate = sf.read(audio)
-                sd.play(data, samplerate)
-                sd.wait()
-                gsm_state.previous_audio = (data, samplerate)
+                data = data.astype('float32')
+                
+                # Use safe audio playback
+                success = play_audio_data_safe(data, samplerate)
+                if success:
+                    gsm_state.previous_audio = (data, samplerate)
             return
+            
         if gsm_state.line_for_screenshot:
             line: GameLine = gsm_state.line_for_screenshot
             gsm_state.line_for_screenshot = None
